{% extends "registration/registration_base.html" %}
{% load i18n %}

{% block title %}{% trans "Password changed" %}{% endblock %}

{% block registration_block %}
<p>{% trans "Password successfully changed!" %}</p>
{% endblock %}


{# This is used by django.contrib.auth #}
