# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-08 15:44
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0028_snapshots_to_iterations'),
    ]

    operations = [
        migrations.AddField(
            model_name='timeentry',
            name='worker',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='time_entries', to='orchestra.Worker'),
        ),
        migrations.AlterField(
            model_name='timeentry',
            name='assignment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='time_entries', to='orchestra.TaskAssignment'),
        ),
    ]
