{% extends 'registration/registration_base.html' %}

{% block title %}Password reset{% endblock %}

{% block registration_block %}

<p>Password reset successfully.</p>
<p><a href="{% url 'auth_login' %}">You may now log in.</a></p>

{% endblock %}
