# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-26 20:34
from __future__ import unicode_literals

import bitfield.models
from django.db import migrations, models
import django.db.models.deletion
import orchestra.models.communication.mixins


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0044_auto_20160426_0044'),
    ]

    operations = [
        migrations.CreateModel(
            name='CommunicationPreference',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('methods', bitfield.models.BitField((('slack', 'Slack'), ('email', 'Email')), blank=True, default=None, null=True)),
                ('communication_type', models.IntegerField(choices=[(0, 'task_status_change')])),
                ('worker', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='orchestra.Worker')),
            ],
            bases=(orchestra.models.communication.mixins.CommunicationPreferenceMixin, models.Model),
        ),
        migrations.AlterUniqueTogether(  # manually-reviewed
            name='communicationpreference',
            unique_together=set([('worker', 'communication_type')]),
        ),
    ]
