/**
 * Base stylesheet for Orchestra.
 *
 * This partial styles the base Django template and serves as the foundation for
 * each Orchestra view.
 */

$header-height: 52px;

html, body {
  font-size: $base-font-size;
}

section.wrapper {
  margin-top: $header-height;
}

section#main-content {
  margin-left: 0;
  color: $dark-gray;
}

header.header {
  background: darken($main-blue, 10);
  box-shadow: 0 1px 3px black;
  min-height: 0;
}

h1, h2, h3, h4, h5, h6, p {
  margin: 0;
  padding: 0
}

a {
  color: $main-blue;
  text-decoration: none;
  &:hover, &:focus, &:visited {
    color: $main-blue;
    text-decoration: underline;
  }
}

a.logo {
  font-size: 2rem;
  color: white;
  margin: 12px 30px;
  &:hover, &:focus, &:visited {
    color: white;
  }
}

.work-timer {
  float: right;
  height: $header-height;
}

#nav-options {
  cursor: pointer;
  font-size: 2rem;
  color: white;
  float: right;
  margin: 12px 30px;
  z-index: 1003;
  .dropdown-menu {
    a {
      color: $main-blue;
      &:hover {
        color: $main-blue;
        text-decoration: underline;
      }
    }
  }
}

#hijacked-warning {
  left: 25%;
  width: 50%;
  border-radius: 0 0 5px 5px;
  > span:last-child {
    // Hack to override button container inline style
    margin-right: 0 !important;
  }
}

.section-panel {
  margin: $panel-button-margin;
  box-shadow: 0 1px 3px black;
  background-color: white;
  .section-header {
    margin-top: 0;
    color: white;
    background-color: $main-blue;
    h3 {
      margin: 1.3rem 0;
      font-size: 1.8rem;
    }
    .btn {
      display: inline-block;
      color: white;
      background-color: $light-green;
      &:first-child {
        margin-left: $panel-button-margin;
      }
      &:hover:not(.disabled) {
        border: 1px solid white;
      }
      &.disabled {
        color: $light-gray;
        background-color: rgba($light-green, 0.6);
      }
    }
  }
  .row:not(.section-header) {
    &.padded {
      > [class*="col-"] {
        padding-top: $panel-padding;
        padding-bottom: $panel-padding;
      }
    }
  }
}

.main-content {
  margin: 100px auto 0;
}
