# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-05-15 15:45
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0061_auto_20160505_2126'),
    ]

    operations = [
        migrations.RenameField(
            model_name='staffingresponse',
            old_name='request',
            new_name='request_inquiry',
        ),
    ]
