# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-26 00:44
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0043_tasktimer_description'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='timeentry',
            options={'verbose_name_plural': 'time entries'},
        ),
    ]
